export declare class SagaInvocationError extends Error {
    readonly originalError: Error;
    readonly step: string;
    constructor(originalError: Error, step: string);
}
export declare class SagaCompensationError extends Error {
    readonly originalError: Error;
    readonly step: string;
    constructor(originalError: Error, step: string);
}
export declare type Fn<I, O = void> = (input: I) => Promise<O> | O;
declare class Step<T> {
    readonly name: string;
    invocation: Fn<T>;
    compensation?: Fn<T>;
    constructor(name: string, invocation: Fn<T>);
    invoke(params: T): Promise<void>;
    compensate(params: T): Promise<void>;
    bind(ctx: unknown): this;
}
declare class SagaFlow<T, R> {
    private readonly steps;
    private readonly returnFn;
    private readonly compensationSteps;
    private __current;
    constructor(steps: Step<T>[], returnFn: Fn<T, R>);
    get current(): Step<T>;
    invoke(params: T): Promise<R>;
    compensate(params: T): Promise<void>;
    bind(ctx: unknown): this;
}
export declare enum SagaStatus {
    New = "New",
    InProgress = "In progress",
    InCompensation = "In compensation",
    Complete = "Complete",
    CompensationComplete = "Compensation complete",
    CompensationError = "Compensation error"
}
export declare class Saga<T, R> {
    private sagaFlow;
    private __status;
    constructor(sagaFlow: SagaFlow<T, R>);
    get status(): SagaStatus;
    execute(params: T): Promise<R>;
    bind(ctx: unknown): this;
    private runCompensationFlow;
}
declare type Cond<IF, THEN, ELSE> = IF extends void ? ELSE : THEN;
declare type BuilderMethodStepResult<T, R> = {
    invoke: (method: Fn<T>) => NextOptions<T, R> & {
        withCompensation: (method: Fn<T>) => NextOptions<T, R>;
    };
};
declare type BuilderMethodReturnResult<T, R> = {
    build: () => Saga<T, R>;
};
declare type NextOptions<T, R> = {
    step: (name?: string) => BuilderMethodStepResult<T, R>;
} & Cond<R, {
    return: (method: Fn<T, R>) => BuilderMethodReturnResult<T, R>;
}, {
    build: () => Saga<T, R>;
}>;
export declare class SagaBuilder<T, R = void> {
    private steps;
    private returnFn;
    step(name?: string): BuilderMethodStepResult<T, R>;
    private return;
    private build;
}
export {};
